function [outinit]=initRIG()
%--------------------------------------------------------------------------
% Initialisation des données du problème :
%
%   1) Paramètres physiques et  numériques
%   2) Géométrie et maillage
%   3) Définitions des solides rigides (ellipses)
%--------------------------------------------------------------------------
% outinit : structure contenant les champs suivants :
%
% 1) Paramètres physiques et numériques :
%
% nu          : viscosite 
% dt          : pas de temps; si dt=[] alors choix automatique de dt 
%               (cf. acfl).
% acfl        : coefficient de la condition anti-CFL, choix automatique de 
%               dt :  dt=acfl*h^(1/3), où h est le pas du maillage.
% cfl         : adaptation automatique du pas de temps dt en fonction de la
%               vitesse; si cfl=[], alors pas d'adaptation. Sinon dt est 
%               choisi tel que : max(|U|)*dt < cfl
%
% nbiter      : nombre d'iterations en temps
%
% type_print  : type d'affichage : 0 -> pas de sortie graphique; 
%                                  1 -> vitesse; 2-> pression
% save_name   : repertoire de sauvegarde pour les resultats
% iterverbose : mode verbeux dans les iterations
% 
%
% 2) Géométrie et maillage
%
% node        : coordonnées [x;y] des noeuds du bord du domaine
% edge        : aretes du bord définies par les numeros des noeuds [n1, n2]
% hdata       : taille(s) du maillage
% optionmesh  : options pour le maillage (dhmax,verbose...)
%
%
% 3) Solide rigide :
%
% fxf, fxs    : force horizontale (en x) dans le fluide et dans le solide
% fyf, fys    : force verticale (en y) dans le fluide et dans le solide
% vcr         : coordonnées (x,y) des points du contour des solides (ellipse)
% xG,yG       : position initiale du centre de masse des solides
% theta       : angle de position initial du solide
% velxG,velyG : vitesse lineaire initiale du solide
% veltheta    : vitesse angulaire initiale du solide
%--------------------------------------------------------------------------
    
    %----------------------------------------------------------------------
    % 1) Paramètres physiques et numériques
    %----------------------------------------------------------------------
    
    nu=0.001;        
    dt=[];
    acfl=0.1;
    cfl=0.5;
    nbiter = 150;  
    type_print=1;           
    save_name='Results'; 
    iterverbose=true; 
    
    %----------------------------------------------------------------------
    % 2) Géométrie : domaine rectangle [0,L]x[0,H] et paramètres du maillage
    %----------------------------------------------------------------------
   
    L=1; H=1;               
    pas_h=0.025;    % Taille du maillage
    
    node = [0 0; L 0; L H; 0 H];
    edge = [1 2; 2 3; 3 4; 4 1];

    hdata = [];
    hdata.hmax  = pas_h;   
    % Raffinement dans une bande (-> cf. fonction refineband.m)
    %hdata.fun = @refineband; hdata.args  = {L, pas_h};
    
    optionmesh=[];
    optionmesh.output=false;  % pas de sortie maillage
    % optionmesh.mlim=0.5;
    % optionmesh.maxit=5;
    % optionmesh.dhmax = 0.25;
    
    %----------------------------------------------------------------------
    % 3) Solide rigide
    %----------------------------------------------------------------------
    
    fxf=0; fxs=0;
    fyf=0; fys=-5; 
    
    rx=0.15;   % rayon en x
    ry=0.15;   % rayon en y
    Ncp=60;    % Nb de points de discretisation sur le bord de l'ellipse
    
    % Positions initiales du solide
    xG=0.5;
    yG=0.7;
    theta=0;
    % Vitesses initiales du solide
    velxG=0;
    velyG=0;
    veltheta=0;
    
    % Construction du contour du rigide
    s=linspace(0,1,Ncp+1);
    x=rx*cos(2*pi*s);
    y=ry*sin(2*pi*s);
    vcr=[x; y]';
    
    %----------------------------------------------------------------------
    
    outinit=struct('nu',nu,'dt',dt,'acfl',acfl,'cfl',cfl,'nbiter',nbiter,...
        'type_print',type_print,'save_name',save_name,'iterverbose',iterverbose,...
        'node',node,'edge',edge,'hdata',hdata,'optionmesh',optionmesh,...
        'fxf',fxf,'fxs',fxs,'fyf',fyf,'fys',fys,'vcr',vcr,'xG',xG,...
        'yG',yG,'theta',theta,'velxG',velxG,'velyG',velyG,'veltheta',veltheta);
    
end
