%==========================================================================
%
%                   INTERACTIONS FLUIDE/RIGIDE(S) 
% 
%    MOUVEMENT D'UN CORPS RIGIDE DANS UN FLUIDE VISQUEUX INCOMPRESSIBLE
%    DE TYPE NAVIER-STOKES (2D)
%
%==========================================================================
% 
% * Elements Finis P2/P1 avec une méthode des caractéristiques pour la 
%   nonlinéarité (ordre 2 en temps).
%
% * Densités fluide/solide identiques (=1) : 
%   -> gestion de la densite par un terme source différent dans le solide 
%      et dans le fluide.
%
% * u=0 sur le bord exterieur du domaine
%
% * Maillage du domaine par MESH2D v2.3 
%   Code Matlab sous Licence GNU-GPL -- Copyright (C) 2007 Darren Engwirda
%
%==========================================================================
%
% Attention : Il faut d'abord compiler le fichier 'convect.cpp' (mexfile)
% dans la librairie 'lib'. Pour cela, se mettre dans le répertoire ./lib 
% et lancer une des deux commandes suivantes :
% 
% Pour les plateformes 32-bits :
% >> mex -v -O convect.cpp
%
% Pour les plateformes 64 bits :
% >> mex -v -O -largeArrayDims convect.cpp
%
%==========================================================================

close all
clear all
 
% Path pour mesh2d (maillage)
addpath '../Mesh2d_v23b/' 

% Path pour la librairie Eléments Finis 'lib'
addpath 'lib/'


fprintf('===========================================================\n');
fprintf('                  INTERACTION FLUIDE/RIGIDE                \n');
fprintf('===========================================================\n');

%--------------------------------------------------------------------------
% 1) Paramètres physiques et  numériques
% 2) Géométrie et maillage
% 3) Définition du solide rigide
%--------------------------------------------------------------------------
% outinit : structure contenant les champs suivants :
%
% 1) Paramètres physiques et numériques :
%
% nu          : viscosite
% dt          : pas de temps; si dt=[] alors choix automatique de dt 
%               (cf. acfl).
% acfl        : coefficient de la condition anti-CFL, choix automatique de 
%               dt : dt=acfl*h^(1/3), où h est le pas du maillage.
% cfl         : adaptation automatique du pas de temps dt en fonction de la
%               vitesse; si cfl=[], alors pas d'adaptation. Sinon dt est 
%               choisi tel que  max(|U|)*dt < cfl
% nbiter      : nombre d'iterations en temps
% dt          : pas de temps
% adaptdt     : true/false -> adaptation automatique du pas de temps dt en
%                             fonction de la vitesse.
% nbiter      : nombre d'iterations en temps
% type_print  : type d'affichage : 0 -> pas de sortie graphique; 
%                                  1 -> vitesse; 2-> pression
% save_name   : repertoire de sauvegarde pour les resultats
% iterverbose : mode verbeux dans les iterations
% 
% 2) Géométrie et maillage
%
% node        : coordonnées [x;y] des noeuds du bord du domaine
% edge        : aretes du bord définies par les numeros des noeuds [n1, n2]
% hdata       : taille(s) du maillage
% optionmesh  : options pour le maillage (dhmax,verbose...)
%
% 3) Solides rigides :
%
% fyf,fys     : force verticale en y dans le fluide et dans le(s) solide(s) 
%               (fx=0 partout)
% vcr         : coordonnées (x,y) des points du contour des solides (ellipse)
% xG,yG       : position initiale du centre de masse des solides
% theta       : angle de position initial du solide
% velxG,velyG : vitesse lineaire initiale du solide
% veltheta    : vitesse angulaire initiale du solide
%--------------------------------------------------------------------------
outinit=initRIG();

nu=outinit.nu;
dt1=outinit.dt;
acfl=outinit.acfl;
cfl=outinit.cfl;
nbiter=outinit.nbiter;
type_print=outinit.type_print;
save_name=outinit.save_name;
iterverbose=outinit.iterverbose;

node=outinit.node;
edge=outinit.edge;
hdata=outinit.hdata; pas_h=hdata.hmax;
optionmesh=outinit.optionmesh;

fxf=outinit.fxf; fxs=outinit.fxs;
fyf=outinit.fyf; fys=outinit.fys;
vcr0=outinit.vcr;
xG1=outinit.xG; yG1=outinit.yG; 
theta1=outinit.theta;
velxG=outinit.velxG; velyG=outinit.velyG; veltheta=outinit.veltheta;

%--------------------------------------------------------------------------
% Maillages P1 et P2
%--------------------------------------------------------------------------

outmesh=meshP1P2(node,edge,hdata,optionmesh);
    
v1=outmesh.v1; t1=outmesh.t1; e1=outmesh.e1;
isupp1=outmesh.isupp1; tsupp1=outmesh.tsupp1;
v2=outmesh.v2; t2=outmesh.t2; e2=outmesh.e2;
isupp2=outmesh.isupp2; tsupp2=outmesh.tsupp2;

ib2=outmesh.ib2; ic2=outmesh.ic2;
ibT=outmesh.ibT; icT=outmesh.icT; 

% nv1, nt1, ne1 : nb de noeuds, de triangles et d'aretes du maillage P1
% nv2, nt2, ne2 : nb de noeuds, de triangles et d'aretes du maillage P2
nv1=size(v1,1); nt1=size(t1,1); ne1=size(e1,1);
nv2=size(v2,1); nt2=size(t2,1); ne2=size(e2,1);

fprintf('Maillage P1 : nombre de noeuds    (nv1) = %d\n',nv1);
fprintf('              nombre de triangles (nt1) = %d\n',nt1);
fprintf('Maillage P2 : nombre de noeuds    (nv2) = %d\n',nv2);
fprintf('              nombre de triangles (nt2) = %d\n',nt2);

% Affichage du maillage

if type_print~=0,
  patch('faces',t1(:,1:3),'vertices',v1(:,1:2),'facecolor','white','edgecolor','blue');
  axis equal; axis off
  drawnow 
end

%--------------------------------------------------------------------------
% Pas de discretisation en temps
%--------------------------------------------------------------------------

pas=max(sqrt(t2(:,5)));   % taille (réelle) du maillage P2
if isempty(dt1),
    dt1=acfl*pas^(1/3);    % pas de discretisation en temps 
end
dt=dt1;

%--------------------------------------------------------------------------
% Construction des matrices de Stokes   
%--------------------------------------------------------------------------

[A,M,SS,C]=matrixStokesRIG(v1,t1,nv2,ib2,icT,nu);

fprintf('Taille de la matrice de Stokes : %d x %d (nnz = %d)\n',size(A), nnz(A));

% Factorisation LU de la matrice A : P*A*Q =L*U

fprintf('Factorisation LU...\n');
tstart=tic;
[LA,UA,Pm,Qm]=lu(A,'vector');
fprintf('Temps de la factorisation LU : %f s\n',toc(tstart));

%--------------------------------------------------------------------------
% Positionnnement initial du solide (ellipse)
%--------------------------------------------------------------------------
vcr=vcr0;
Rr=[cos(theta1) sin(theta1); -sin(theta1) cos(theta1)];
vcr(:,1)=xG1+vcr0(:,1:2)*Rr(:,1);
vcr(:,2)=yG1+vcr0(:,1:2)*Rr(:,2);     

% Détermination des noeuds dans le solide (is) et dans le fluide (if)

[is1,is2,if1,if2]=nodesRIG(v1,v2,vcr,t1,t2,isupp1,tsupp1,isupp2,tsupp2,ib2);

% -------------------------------------------------------------------------
% Construction de la vitesse initiale :
%
% vitesse initiale = 0 dans le fluide; 
%                  = vitesse rigide : (velxG,velyG,veltheta) dans le rigide 
% -------------------------------------------------------------------------

Mutr=MatrixMutvect(xG1,yG1,nv1,nv2,if1,if2,is1,is2,ibT,v2);

uinit=[sparse(size(Mutr,2)-3,1); velxG; velyG; veltheta];
uinit=Mutr*uinit;
u=zeros(2*nv2+nv1,1);
u(icT)=uinit;
u(ibT)=0;     % u=0 sur le bord extérieur

U1x=u(1:nv2);
U1y=u((nv2+1):(2*nv2));
P=u((2*nv2+1):(2*nv2+nv1));

U2x=U1x;U2y=U1y;

%--------------------------------------------------------------------------
% Sauvegarde des fichiers
%--------------------------------------------------------------------------

[success,message]=mkdir(save_name);
nom_fich=strcat(save_name,'/Rigid_numdata.dat');
save(nom_fich,'nbiter','pas_h','nu','-ASCII');
nom_fich=strcat(save_name,'/Rigid_v2.dat');
save(nom_fich,'v2');
nom_fich=strcat(save_name,'/Rigid_t2.dat');
save(nom_fich,'t2')
nom_fich=strcat(save_name,'/Rigid_v1.dat');
save(nom_fich,'v1')
nom_fich=strcat(save_name,'/Rigid_t1.dat');
save(nom_fich,'t1')
nom_fich=strcat(save_name,'/Rigid_vcr0.dat');
save(nom_fich,'vcr0')
nom_fich=strcat(save_name,'/Rigid_Ux_0000.dat');
Ux=U1x;Uy=U1y;
save(nom_fich,'Ux')
nom_fich=strcat(save_name,'/Rigid_Uy_0000.dat');
save(nom_fich,'Uy')
nom_fich=strcat(save_name,'/Rigid_P_0000.dat'); 
save(nom_fich,'P');

nom_fich_position=strcat(save_name,'/','Rigid_position.dat');
fid = fopen(nom_fich_position,'w+t');
fprintf(fid,'%6.3e %6.3e %6.3e\n',xG1,yG1,theta1);
fclose(fid);

nom_fich_vitesse=strcat(save_name,'/','Rigid_vitesse.dat');
fid = fopen(nom_fich_vitesse,'w+t');
fprintf(fid,'%6.3e %6.3e %6.3e\n',velxG,velyG,veltheta);
fclose(fid);

nom_fich_dt=strcat(save_name,'/','Rigid_dt.dat');
fid = fopen(nom_fich_dt,'w+t');
if isempty(cfl)
    fprintf(fid,'%6.3e\n',dt);
end
fclose(fid);

fid_pos = fopen(nom_fich_position,'at');
fid_vel = fopen(nom_fich_vitesse,'at');
fid_dt = fopen(nom_fich_dt,'at');

%--------------------------------------------------------------------------
% Calcul NS+rigide : Iterations en temps
%--------------------------------------------------------------------------
fprintf('-----------------------------------------------------------\n');
fprintf('Itérations en temps                                        \n');
fprintf('-----------------------------------------------------------\n');
if isempty(cfl),
    fprintf('Pas de temps dt=%e\n',dt);
else
    fprintf('Pas de temps adaptatif\n'); 
end
fprintf('Nombre d''iterations en temps : %d\n',nbiter);

if ~iterverbose,
    fprintf('Progression des itérations (1...%d)=1',min(20,nbiter));
    htt=(nbiter-1)/19;
    jtt=1;
end

touchbd=false;   % -> pour le test si le rigide touche le bord

for it=1:nbiter
    
    time1=tic;
        
    % Pas de temps adaptatif
    if ~isempty(cfl)
        normv=sqrt( (velxG-veltheta*vcr0(:,2)).^2+(velyG+veltheta*vcr0(:,1)).^2 );
        if max(normv)==0,
             dt=dt1;
        else
             dt=min(dt1,cfl*pas/max(normv));
        end
    end
    
    % Affichage
    if type_print~=0,
        clf;
        plotsolRIG(U2x,U2y,P,t1,t2,v1,v2,vcr,(it-1)*dt,it,type_print)
    end
        
    if iterverbose,
      fprintf('--------------------------------------------\n');
      fprintf(' Itération : %d\n',it);
    elseif floor(1+jtt*htt)==it || nbiter<20
            jtt=jtt+1; fprintf('%d',jtt);            
    end

    %----------------------------------------------------------------------
    % Mouvement du rigide de tk a tk+dt
    %----------------------------------------------------------------------
    xG2=xG1+velxG*dt;
    yG2=yG1+velyG*dt;
    theta2=theta1+veltheta*dt;
    
    if iterverbose,
      fprintf('velxG = %f, velyG = %f, veltheta=%f\n',velxG,velyG,veltheta);
      fprintf('G=(%f,%f), theta=%f\n',xG2,yG2,theta2);
    end
    
    % Changement du repere du rigide au repere global
    Rr=[cos(theta2) sin(theta2);-sin(theta2) cos(theta2)];
    vcr(:,1)=xG2+vcr0(:,1:2)*Rr(:,1);
    vcr(:,2)=yG2+vcr0(:,1:2)*Rr(:,2);

    % Détermination des noeuds dans le solide (is) et dans le fluide (if)
    [is1,is2,if1,if2]=nodesRIG(v1,v2,vcr,t1,t2,isupp1,tsupp1,...
                               isupp2,tsupp2,ib2);
    
    nf1=length(if1); nf2=length(if2);
    
    %----------------------------------------------------------------------
    % Terme source
    %----------------------------------------------------------------------
    fx=fxf*ones(nv2,1);
    fy=fyf*ones(nv2,1);
    fx(is2)=fxs;
    fy(is2)=fys;
    
    %----------------------------------------------------------------------
    %  Matrice de passage MuTranspose
    %----------------------------------------------------------------------
    tstart=tic;
    
    Mutr=MatrixMutvect(xG2,yG2,nv1,nv2,if1,if2,is1,is2,ibT,v2);
    
    if iterverbose,
      fprintf('Temps d''assemblage de la matrice MU : %f s\n',toc(tstart));
    end
    
    %----------------------------------------------------------------------
    % Calcul de la caractéristique
    %----------------------------------------------------------------------
    tstart=tic; 
    if it==1
                
        % Convection d'ordre 1 en temps
        V=convect(length(U1x),full(U1x),full(U1y),full(U1x),full(U1y),...
                  dt,v2(:,1:2),t2(:,1:3),isupp2',tsupp2);    
        Uxm=V(1:nv2);                                               
        Uym=V(nv2+1:end);
    
        %----------------------------------------------------------------------
        % Second membre et matrice complète du système pour it=1
        %----------------------------------------------------------------------
        
        ld=[SS*Uxm;SS*Uym;sparse(nv1,1)]/dt;
        
        Ag=Mutr'*(A+M/dt)*Mutr;
        
    else
        
        % Convection d'ordre 2 en temps
        Vx=2*U2x-U1x;
        Vy=2*U2y-U1y;
                       
        V=convect(length(U1x),full(U1x),full(U1y),full(Vx),full(Vy),...
                  2*dt,v2(:,1:2),t2(:,1:3),isupp2',tsupp2);
        U1xm=V(1:nv2);
        U1ym=V(nv2+1:end);
        
        V=convect(length(U2x),full(U2x),full(U2y),full(Vx),full(Vy),...
                  dt,v2(:,1:2),t2(:,1:3),isupp2',tsupp2);
        U2xm=V(1:nv2);
        U2ym=V(nv2+1:end);
        
        %------------------------------------------------------------------
        % Second membre et matrice complète du système pour it>1
        %------------------------------------------------------------------
        
        ld=[SS*(2*U2xm-0.5*U1xm);SS*(2*U2ym-0.5*U1ym);sparse(nv1,1)]/dt;
        Ag=Mutr'*(A+3*M/(2*dt))*Mutr;
        
    end
    
    if iterverbose,
          fprintf('Temps de calcul de la convection : %f s\n',toc(tstart));
    end
    
    % Ajout du terme source dans le second memebre
    ld=ld+[SS*fx;SS*fy;sparse(nv1,1)];
    ld(ibT)=[];
    
    Bg=Mutr'*ld;
     
    % Introduction du multiplicateur de Lagrange pour la pression : 
    %Bg=[Bg;0];
    %Cg=Mutr'*C; 
    %Ag=[Ag Cg;Cg' 0];
        
    % Traitement des points du rigide qui touchent le bord du domaine
    if ~touchbd,
        [~,in1,in2]=intersect(is2,ib2); 
        if length(in1)>1, touchbd=true; end;
    end
    
    if touchbd
        I=2*nf2+nf1+1:2*nf2+nf1+3;
        Ag(I,:)=0; Ag(:,I)=0; Ag(I,I)=1;
        Bg(I)=0;
    end;

    %----------------------------------------------------------------------
    % Résolution du système linéaire
    %----------------------------------------------------------------------
    
    %tstart=tic;
    %r=amd(Ag); 
    %spparms('autommd',0); spparms('autoamd',0); 
    %solSS=Ag(r,r)\Bg(r);
    %solSS(r)=solSS;
    %toc(tstart)
    %spparms('autommd',1); spparms('autoamd',1); 
    
    tstart=tic;
    solS=Ag\Bg;
    
    if iterverbose,
      fprintf('Temps de résolution du système linéaire : %f s\n',toc(tstart));
    end
    
    % Récupération de la vitesse et de la pression

    velxG=full(solS(2*nf2+nf1+1));
    velyG=full(solS(2*nf2+nf1+2));
    veltheta=full(solS(2*nf2+nf1+3));
    
    I=1:(2*nf2+nf1+3);
    sol=Mutr(:,I)*solS(I);

    xG1=xG2; yG1=yG2;
    theta1=theta2;
    U1x=U2x;U1y=U2y;
    
    u(icT)=sol;
    u(ibT)=0;   % u=0 sur le bord extérieur
    U2x=u(1:nv2);
    U2y=u((nv2+1):(2*nv2));
    P=u((2*nv2+1):(2*nv2+nv1));
    
    %----------------------------------------------------------------------
    % Sauvegarde fichiers
    %----------------------------------------------------------------------
       
    if ~isempty(cfl)
        fprintf(fid_dt,'%6.3e\n',dt);
    end
    
    Ux=U2x; Uy=U2y;
    nom_fich=sprintf('%s/Rigid_Ux_%.4d.dat',save_name,it);
    save(nom_fich,'Ux')
    nom_fich=sprintf('%s/Rigid_Uy_%.4d.dat',save_name,it);
    save(nom_fich,'Uy')
    nom_fich=sprintf('%s/Rigid_P_%.4d.dat',save_name,it);
    save(nom_fich,'P')
    xG=xG1; yG=yG1;theta=theta1;
    fprintf(fid_pos,'%6.3e %6.3e %6.3e\n',xG,yG,theta);
    fprintf(fid_vel,'%6.3e %6.3e %6.3e\n',velxG,velyG,veltheta);
    
    if iterverbose,
      fprintf('Temps de calcul pour l''iteration %d : %f s\n',it,toc(time1));
    end
end

fprintf('\n');
fclose(fid_pos);
fclose(fid_vel);
fclose(fid_dt);
